/*
** Module   : tty.keyboard.js
** Abstract : keystrokes management.
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ----------------------Description--------------------------
** 001 MAG 20140110 First version.
*/

tty.keyboard = (function(socket, map, tty)
{
   /* exports object */
   var my = {}; 
   
   /* track escape status */
   var escape = false;
                     
   /* track escape status */
   var altChar = false;

   /* initialize module */
   my.init = function()
   {      
      /* init mapping */   
      tty.keymap.init();
         
      /* keypress */
      document.addEventListener("keypress", function(evt)
      {
         var key = evt.charCode;
         if (key != 0)
         {
            sendKey(key, evt);
         }
      }, false);
                 
      /* keyup */
      document.addEventListener("keyup", function(evt)
      {
         if (evt.keyCode === keys.ALT && evt.location === evt.DOM_KEY_LOCATION_RIGHT)
         {
            altChar = false;
         }
      }, false);

      /* keydown */
      document.addEventListener("keydown", function(evt)
      {
         var key = evt.keyCode;

         if(altChar || key === keys.CTRL  || key === keys.SHIFT)
         {
            return;
         }

         if (key == keys.ALT)
         {
            if (evt.location === evt.DOM_KEY_LOCATION_RIGHT)
            {
               altChar = evt.ctrlKey;                  
            }
            return;
         }
         
         if (key === keys.ESCAPE)
         {
            escape = !escape;
            return;               
         }
                                  
         var p4code = -1;
         var charCode = map.mapKey(evt);
                                          
         // Handle ALT or Escape
         if (evt.altKey || escape)
         {
            p4code = map.xlateAlt[key];
            
            if (p4code === undefined)
            {
               if (charCode != -1 && ((!escape && evt.altKey) || (escape && !evt.altKey)))
               {
                  p4code = map.processCtrl(evt, charCode);
                  if(p4code == -1)
                  {
                     p4code = charCode;
                  }
                  p4code |= 1024;
               }
               else
               {
                  p4code = -1;
               }
            }
            else
            {
               if (!escape && evt.altKey && key == keys.DELETE)
               {
                  // special case for ALT+DELETE key combo
                  p4code = 512;
               }     
            }    
                       
            escape = false;
         }
         // Normal
         else
         {
            /* normal map */
            var p4code = map.xlate[key];
            
            if (p4code === undefined)
            {
               // check CTRL
               p4code = map.processCtrl(evt, charCode);
            }
            else
            {
               if (p4code == keys.TAB && evt.shiftKey)
               {
                  p4code = 509;
               }                           
            }
         }                                                           

         if (p4code != -1)
         {
            sendKey(p4code, evt);
         }
         
      }, false);//on key
                                                                                 
   };//init
         
   /* send keyEvent */
   var sendKey = function(key, evt)
   {
      var data = {};
      data.c = 0;
      data.k = key;
      socket.send(JSON.stringify(data));
      evt.preventDefault();
      evt.stopImmediatePropagation();
   };
            
   /* exports */
   return my;    

}(tty.socket, tty.keymap, tty));
